if($Server::HomingWeaponsVersion > 1)
   return;
$Server::HomingWeaponsVersion = 1;
$Server::HomingWeaponsDebug = 0;

if ($Server::HomingWeaponsDebug)
{
	echo("Support_HomingWeapons.cs - DEBUG MODE IS ON!");
}

package HomingWeapons
{
	function Projectile::onAdd(%obj,%a,%b)
	{
		Parent::onAdd(%obj,%a,%b);
		%data = %obj.getDatablock();
		if(%data.homing || %obj.homing)
		{
			%data.startHomingLoop(%obj);
		}
	}
	
	//Trying to fix multi hit bug
	function ProjectileData::onCollision(%this, %obj, %col, %fade, %pos, %normal)
	{
		Parent::onCollision(%this, %obj, %col, %fade, %pos, %normal);
		if(%this.homing || %obj.homing)
		{
			%this.stopHomingLoop(%obj);
		}
	}
	
	function ProjectileData::onExplode(%this, %obj, %pos, %fade)
	{
		Parent::onExplode(%this, %obj, %pos, %fade);
		if(%this.homing || %obj.homing)
		{
			%this.stopHomingLoop(%obj);
		}
	}
};
activatePackage(HomingWeapons);

//Starts the homing loop
function ProjectileData::startHomingLoop(%data, %obj)
{
	if ($Server::HomingWeaponsDebug)
		echo("Support_HomingWeapons.cs - startHomingLoop has been entered with the following parameters:" SPC %data SPC "," SPC %obj);
	
	if (%obj.doneHomingDelay)
	{
		%homingStartDelay = %data.homingDelay;
		if (%homingStartDelay $= "")
			%homingStartDelay = 100;
	}
	else
	{
		%homingStartDelay = %data.homingStartDelay;
		if (%homingStartDelay $= "")
		{
			%homingStartDelay = %data.homingDelay;
			if (%homingStartDelay $= "")
				%homingStartDelay = 300;
		}
	}
	
	%obj.homing = 1;
	%obj.doneHomingDelay = 1;
	%obj.homeLoop = %data.schedule(%homingStartDelay,homingLoop, %obj);
}

//Stops the homing loop
function ProjectileData::stopHomingLoop(%data, %obj)
{
	if ($Server::HomingWeaponsDebug)
		echo("Support_HomingWeapons.cs - stopHomingLoop has been entered with the following parameters:" SPC %data SPC "," SPC %obj);
	
	%obj.homing = 0;
	cancel(%obj.homeLoop);
}

//The actual homing loop
function ProjectileData::homingLoop(%data, %obj)
{
	if(!isObject(%obj.client))
		return;
	
	if(!isObject(%obj) || !%obj.homing || vectorLen(%obj.getVelocity()) == 0)
		return;
		
	%client = %obj.client;
	
	if (!%data.isValidTarget(%obj.target))
	{
		//We have to find a new target
		%found = %data.findHomeTarget(%obj);
		
		if(isObject(%found))
			%obj.target = %found;
		else
		{
			%homingDelay = %data.homingStartDelay;
			if (%homingDelay $= "")
			{
				%homingDelay = %data.homingDelay;
				if (%homingDelay $= "")
					%homingDelay = 300;
			}
			%obj.homeLoop = %data.schedule(%homingDelay,homingLoop, %obj);
			return;
		}
	}
	
	%data.homeToTarget(%obj, %obj.target);
	
}

//Finds a target for the homing projectile
//Returns Target of one of the following types: Vector, Vehicle, Player/Bot, Brick, Projectile
//	Return null if no target has been found.
function ProjectileData::findHomeTarget(%data, %obj)
{
	if ($Server::HomingWeaponsDebug)
		echo("Support_HomingWeapons.cs - findHomeTarget ENTER" SPC %data SPC "," SPC %obj);
	
	%found = null;
	%pos = %obj.getPosition();
	%radius = %data.homingSearchRadius;
	if (%radius $= "")
	{
		%radius = %data.homingMaxDistance;
		if (%radius $= "")
			%radius = 100;
	}
	%searchMasks = $TypeMasks::PlayerObjectType;
	InitContainerRadiusSearch(%pos, %radius, %searchMasks);
	%minDist = %radius * 10;
	while ((%searchObj = containerSearchNext()) != 0 )
	{
		if((miniGameCanDamage(%obj.sourceObject,%searchObj)) == 1)
		{
			if(%searchObj.getState() $= "Dead")
				continue;
			
			//if(%client == %searchObj.client)
			if(%obj.sourceObject == %searchObj)
				continue;
			
			if(isTeamFriendly(%obj,%searchObj) != 0)
				continue;
			
			if(%searchObj.getMountedImage(0) == adminWandImage.getID())
				continue;
			
			if(%searchObj.isCloaked)
				continue;
			
			%d = vectorDist(%pos,%searchObj.getPosition());
			if(%d < %minDist)
			{
				%minDist = %d;
				%found = %searchObj;
			}
		}
	}
	
	//Always return
	if ($Server::HomingWeaponsDebug)
		echo("Support_HomingWeapons.cs - findHomeTarget RETURN" SPC %found);
	return %found;
}

//Handles the movement of the projectile towards the given target.
function ProjectileData::homeToTarget(%data, %obj, %target)
{
	if ($Server::HomingWeaponsDebug)
		echo("Support_HomingWeapons.cs - homeToTarget ENTER" SPC %data SPC "," SPC %obj SPC "," SPC %target);
	
	%pos = %obj.getPosition();
	%muzzle = vectorLen(%obj.getVelocity());
	%start = %pos;
	%end = %target.getHackPosition();
	%enemypos = %end;
	%vec = vectorNormalize(vectorSub(%end,%start));
	//Maybe we can leave this out for simple homing to reduce load on server?
	if (%data.homingPredict)
	{
		for(%i=0;%i<5;%i++)
		{
			%t = vectorDist(%start,%end) / vectorLen(vectorScale(getWord(%vec,0) SPC getWord(%vec,1),%muzzle));
			%velaccl = vectorScale(%accl,%t);
			
			%x = getword(%velaccl,0);
			%y = getword(%velaccl,1);
			%z = getWord(%velaccl,2);
			
			%x = (%x < 0 ? 0 : %x);
			%y = (%y < 0 ? 0 : %y);
			%z = (%z < 0 ? 0 : %z);
			
			%vel = vectorAdd(vectorScale(%target.getVelocity(),%t),%x SPC %y SPC %z);
			%end = vectorAdd(%enemypos,%vel);
			%vec = vectorNormalize(vectorSub(%end,%start));
		}
	}
	
	%turningSpeed = %data.homingTurn;
	if (%turningSpeed $= "")
		%turningSpeed = 0.5;
	%addVec = vectorAdd(%obj.getVelocity(),vectorScale(%vec,(360 * %turningSpeed)/vectorDist(%pos,%end)*(%muzzle/%data.muzzleVelocity)));
	%vec = vectorNormalize(%addVec);
	
	%p = new Projectile()
	{
		dataBlock = %obj.dataBlock;
		initialPosition = %pos;
		initialVelocity = vectorScale(%vec,%muzzle);
		sourceObject = %obj.sourceObject;
		client = %obj.client;
		sourceSlot = 0;
		originPoint = %obj.originPoint;
		doneHomingDelay = 1;
		target = %obj.target;
		reflectTime = %obj.reflectTime;
	};
	
	if(isObject(%p))
	{
		MissionCleanup.add(%p);
		%p.setScale(%obj.getScale());
		%obj.delete();
	}
}

//Validates if the given target can be homed towards
function ProjectileData::isValidTarget(%data, %obj, %target)
{
	if ($Server::HomingWeaponsDebug)
		echo("Support_HomingWeapons.cs - canHomeToTarget ENTER" SPC %data SPC "," SPC %obj SPC "," SPC %target);
	
	if (!isObject(%target))
	{
		return 0;
	}
	if (%target.getState() $= "Dead")
		return 0;
	
	if (%target.getMountedImage(0) == adminWandImage.getID())
		return 0;
	
	%homingDistance = %data.homingMaxDistance;
	if (%homingDistance $= "")
		%homingDistance = 30;
	if (vectorDist(%obj.getPosition(),%obj.target.getHackPosition()) > %homingDistance)
		return 0;
	
	
	//If everything is okay, return true.
	if ($Server::HomingWeaponsDebug)
		echo("Support_HomingWeapons.cs - canHomeToTarget SUCCES");
	return 1;
}

function isTeamFriendly(%obj1,%obj2)
{
	// 0 = enemy
	// 1 = allied
	//-1 = neutral
	%mini1 = getMinigameFromObject(%obj1);
	%mini2 = getMinigameFromObject(%obj2);
	if(%mini1 != %mini2)
		return -1;
	
	if(!isObject(%mini1))
		return -1;
	
	if(%obj1 == %obj2)
		return 1;
	
	//With Team Deathmatch, players on different TDM teams are enemies, as well as those in team -1 (neutral)
	%usingTDM = (isObject(GameModeStorerSO) && GameModeStorerSO.modeUsesTeams[%mini1.gamemode]);
	
	//With either zombie mod, players that are on a zombies team and players that aren't are enemies (handily both are "player.isZombie")
	
	//Iban's zombies will never run with Team Deathmatch
	%usingIbanZombies = ($ZAPT::Version > 0 && miniGameUsingTDMGameMode(%mini1) && %mini1.ZAPT_enabled);
	
	//Rotondo's zombies can - if enabled, players on a neutral TDM team are allied.
	%usingRotZombies = ($pref::Server::ZombiesEnabled && %mini1.EnableZombies);
	
	switch$(%obj1.getClassName())
	{
		case "Player":
			%cl1 = %obj1.client;
			%zombie1 = (%usingIbanZombies && %obj1.isZombie);
		case "AIPlayer":
			%zombie1 = %obj1.isZombie;
			if(isObject(%obj1.client))
				%cl1 = %obj1.client;
			else if(isObject(%obj1.spawnBrick) && %usingTDM)
				%team1 = %obj1.spawnBrick.tdmAlliedTeam-1;
		case "Projectile":
			%zombie1 = (isObject(%obj1.sourceObject) && (%obj1.sourceObject.getType() & $TypeMasks::PlayerObjectType) && %obj1.sourceObject.isZombie);
			%cl1 = %obj1.client;
		case "GameConnection":
			%zombie1 = (%obj1.player.isZombie);
			%cl1 = %obj1;
		case "AIConnection":
			%zombie1 = (%obj1.player.isZombie);
			%cl1 = %obj1;
		default:
			return -1;
	}
	
	if(%usingTDM && %tdmTeam1 $= "" && isObject(%cl1))
		%team1 = %cl1.tdmTeam;
	
	switch$(%obj2.getClassName())
	{
		case "Player":
			%cl2 = %obj2.client;
			%zombie2 = (%usingIbanZombies && %obj2.isZombie);
		case "AIPlayer":
			%zombie2 = %obj2.isZombie;
			if(isObject(%obj2.client))
				%cl2 = %obj2.client;
			else if(isObject(%obj2.spawnBrick) && %usingTDM)
				%team2 = %obj2.spawnBrick.tdmAlliedTeam-1;
		case "Projectile":
			%zombie2 = (isObject(%obj2.sourceObject) && (%obj2.sourceObject.getType() & $TypeMasks::PlayerObjectType) && %obj2.sourceObject.isZombie);
			%cl2 = %obj2.client;
		case "GameConnection":
			%zombie2 = (%obj2.player.isZombie);
			%cl2 = %obj2;
		case "AIConnection":
			%zombie2 = (%obj2.player.isZombie);
			%cl2 = %obj2;
		default:
			return -1;
	}
	
	if(%usingTDM && %tdmTeam2 $= "" && isObject(%cl2))
		%team2 = %cl2.tdmTeam;
	
	//echo(%team1 SPC %team2);
	
	if(%usingTDM && %team1 == %team2)
	{
		if(%team1 == -1)
			return %usingRotZombies;
		
		return 1;
	}
	
	if(%usingIbanZombies || %usingRotZombies)
		return (%zombie1 == %zombie2);
	
	return 0;
}
